@@ -59,23 +59,37 @@ def _func_R_y(psi):
 
 @jit(nopython=True,cache=True,nogil=True)
 def _directional_cosine(ra_dec):
-    #The coordinate definition of the measurement set V2/3 is used (https://casacore.github.io/casacore-notes/264.pdf see page 12 and https://drive.google.com/file/d/1a-eUwNrfnYjaUQTjJDfOjJCa8ZaSzZcn/view?usp=sharing).
-    #Note this does not follow the convention in chapter 4 of https://link.springer.com/book/10.1007/978-3-319-44431-4 or CASACORE.
-    #Celestial coordinate system is defined by:
-    #        Z pointing to NCP (dec = pi/2),
-    #        X pointing to the East (ra = pi/2, dec = 0),
-    #       -Y pointing to the vernal equinox (ra = 0, dec = 0).
-    # The uvw coordinates are aligned with XYZ when the source is located at the NCP (ra=0 and dec=pi/2).
-    # A good overview of coordinates systems [1] http://www.danfleisch.com/maxwell/CoordinateSystemReview.pdf
-    # Using the physics convention for spherical coordinates:
-    #        Polar angle      theta = pi/2 - dec (1)
-    #        Azimuthal Angle  phi   = 3*pi/2 + ra (2)
-    #Substituting (1) and (2) into equations (8)-(10) of [1] yields:
-    lmn = np.zeros((3,))
-    lmn[0] = np.sin(ra_dec[0])*np.cos(ra_dec[1])
-    lmn[1] = -np.cos(ra_dec[0])*np.cos(ra_dec[1])
-    lmn[2] = np.sin(ra_dec[1])
-   
+    """
+    The coordinate definition of the measurement set V2/3 is used (https://casacore.github.io/casacore-notes/264.pdf see page 12 and https://drive.google.com/file/d/1a-eUwNrfnYjaUQTjJDfOjJCa8ZaSzZcn/view?usp=sharing). Note this does not follow the convention in chapter 4 of https://link.springer.com/book/10.1007/978-3-319-44431-4 or CASACORE.
+    
+    Celestial coordinate system is defined by:
+       - Z pointing to NCP (dec = pi/2),
+       - X pointing to the East (ra = pi/2, dec = 0),
+       - (-Y) pointing to the vernal equinox (ra = 0, dec = 0).
+    The uvw coordinates are aligned with XYZ when the source is located at the NCP (ra=0 and dec=pi/2).
+    A good overview of coordinates systems [1] http://www.danfleisch.com/maxwell/CoordinateSystemReview.pdf
+    Using the physics convention for spherical coordinates:
+       - Polar angle      theta = pi/2 - dec (1)
+       - Azimuthal Angle  phi   = 3*pi/2 + ra (2)
+    Substituting (1) and (2) into equations (8)-(10) of [1] yields lmn.
+    
+    Parameters
+    ----------
+    ra_dec: float np.array, [n_pos,2], radians
+        Right ascension and declination coordinates.
+    Returns
+    -------
+    lmn : float np.array, [n_pos,2], radians
+        Directional cosine coordinates.
+    """
+    ra_dec_shape = ra_dec.shape
+    lmn = np.zeros((ra_dec_shape[0],3),dtype = numba.float64)
+    
+    for i in range(ra_dec_shape[0]):
+        lmn[i,0] = np.sin(ra_dec[i,0])*np.cos(ra_dec[i,1])
+        lmn[i,1] = -np.cos(ra_dec[i,0])*np.cos(ra_dec[i,1])
+        lmn[i,2] = np.sin(ra_dec[i,1])
+
     return lmn
     
 @jit(nopython=True,cache=True,nogil=True)
@@ -88,21 +102,23 @@ def _sin_project(ra_dec_o,ra_dec):
     ----------
     ra_dec_o: float np.array, [2], radians
         Right ascension and declination of the image center on the celestial sphere.
-    ra_dec: float np.array, [2], radians
+    ra_dec: float np.array, [n_pos,2], radians
         Right ascension and declination of a point on the celestial sphere.
     Returns
     -------
-    lm : float np.array, [2], radians
+    lm : float np.array, [n_pos,2], radians
         Right ascension and declination (ra_dec) projected onto the image plane.
     """   
-    
-    ra = ra_dec[0]
-    dec = ra_dec[1]
+    ra_dec_shape = ra_dec.shape
+    lm = np.zeros(ra_dec_shape,dtype = numba.float64)
     ra_o = ra_dec_o[0]
     dec_o = ra_dec_o[1]
-    lm = np.zeros((2,),dtype = numba.float64)
-    lm[0] = np.cos(dec)*np.sin(ra-ra_o)
-    lm[1] = np.sin(dec)*np.cos(dec_o) - np.cos(dec)*np.sin(dec_o)*np.cos(ra-ra_o)
+    
+    for i in range(ra_dec_shape[0]):
+        ra = ra_dec[i,0]
+        dec = ra_dec[i,1]
+        lm[i,0] = np.cos(dec)*np.sin(ra-ra_o)
+        lm[i,1] = np.sin(dec)*np.cos(dec_o) - np.cos(dec)*np.sin(dec_o)*np.cos(ra-ra_o)
     return lm
     
 @jit(nopython=True,cache=True,nogil=True)
@@ -114,22 +130,25 @@ def _inv_sin_project(ra_dec_o,lm):
     ----------
     ra_dec_o: float np.array, [2], radians
         Right ascension and declination of the image center on the celestial sphere.
-    lm : float np.array, [2], radians
+    lm : float np.array, [n_pos,2], radians
         Coordinates of a point on the image plane.
     Returns
     -------
-    ra_dec: float np.array, [2], radians
+    ra_dec: float np.array, [n_pos,2], radians
         Image plane coordinates (lm) projected onto the celestial sphere.
     """
-    
+    lm_shape = lm.shape
+    ra_dec = np.zeros(lm_shape,dtype = numba.float64)
+    #ra_dec = np.zeros(lm_shape)
     ra_o = ra_dec_o[0]
     dec_o = ra_dec_o[1]
-    ra_dec = np.zeros((2,),dtype = numba.float64)
-    l = lm[0]
-    m = lm[1]
-    n = np.sqrt(1 - l**2 - m**2)
-    ra_dec[0] = ra_o + np.arctan2(l,n*np.cos(dec_o)-m*np.sin(dec_o))
-    ra_dec[1] = np.arcsin(m*np.cos(dec_o) + n*np.sin(dec_o))
+    
+    for i in range(lm_shape[0]):
+        l = lm[i,0]
+        m = lm[i,1]
+        n = np.sqrt(1 - l**2 - m**2)
+        ra_dec[i,0] = ra_o + np.arctan2(l,n*np.cos(dec_o)-m*np.sin(dec_o))
+        ra_dec[i,1] = np.arcsin(m*np.cos(dec_o) + n*np.sin(dec_o))
     return ra_dec
     
 def _sin_pixel_to_celestial_coord(ra_dec_o,image_size,delta,pixel):
@@ -138,17 +157,17 @@ def _sin_pixel_to_celestial_coord(ra_dec_o,image_size,delta,pixel):
     
     Parameters
     ----------
-    ra_dec_o: float np.array, [2], radians
+    ra_dec_o: float np.array, [n_pos,2], radians
         Array with the right ascension and declination of the image center on the celestial sphere.
     image_size : int np.array, [2]
         Size of image plane.
     delta: float np.array, [2], radians
         The size of a single pixel.
-    pixel: float np.array, [2]
+    pixel: float np.array, [n_pos,2]
         Pixel coordinate on the grid defined by image_size and delta. Fractional pixel coordinates are allowed.
     Returns
     -------
-    ra_dec: float np.array, [2], radians
+    ra_dec: float np.array, [n_pos,2], radians
         Image plane coordinates (lm) projected onto the celestial sphere.
     """
     lm = (pixel - image_size//2)*delta
@@ -167,11 +186,11 @@ def _celestial_coord_to_sin_pixel(ra_dec_o,image_size,delta,ra_dec):
         Size of image plane.
     delta: float np.array, [2], radians
         The size of a single pixel.
-    ra_dec: float np.array, [2], radians
+    ra_dec: float np.array, [np_pos,2], radians
         Right ascension and declination of a point on the celestial sphere.
     Returns
     -------
-    pixel: float np.array, [2]
+    pixel: float np.array, [n_pos,2]
         Pixel coordinate on the grid defined by image_size and delta.
     """
     lm = _sin_project(ra_dec_o,ra_dec)
@@ -207,16 +226,16 @@ def _local_spherical_coords(ra_dec_o,ra_dec):
     
 
 @jit(nopython=True,cache=True,nogil=True)
-def _calc_rotation_mats(ra_dec_in,ra_dec_out):
+def _calc_rotation_mats(ra_dec_o,ra_dec):
     
     #The default
     #Rotates input system to output system.
-    uvw_rotmat = _func_R_x(-(np.pi/2-ra_dec_in[1]))@_func_R_z(ra_dec_out[0]-ra_dec_in[0])@_func_R_x(np.pi/2 - ra_dec_out[1])
+    uvw_rotmat = _func_R_x(-(np.pi/2-ra_dec_o[1]))@_func_R_z(ra_dec[0]-ra_dec_o[0])@_func_R_x(np.pi/2 - ra_dec[1])
     
     #Rotates output to Standard System
-    out_rotmat = _func_R_x(-(np.pi/2-ra_dec_out[1]))@_func_R_z(-ra_dec_out[0])
-    lmn_out = _directional_cosine(ra_dec_out)
-    lmn_in = _directional_cosine(ra_dec_in)
+    out_rotmat = _func_R_x(-(np.pi/2-ra_dec[1]))@_func_R_z(-ra_dec[0])
+    lmn_out = _directional_cosine(ra_dec.reshape(1,-1))[0,:]
+    lmn_in = _directional_cosine(ra_dec_o.reshape(1,-1))[0,:]
     
     lmn_rot = out_rotmat@(lmn_out - lmn_in) # out_rotmat(lmn_out - lmn_in)= [0,0,1] - out_rotmat@[l_in,m_in,n_in]
     
